local beql = beql
local L = AceLibrary("AceLocale-2.2"):new("beql")

function beql:InitTracker()
	MAX_QUESTWATCH_LINES = 50
	MAX_WATCHABLE_QUESTS = 20
	for i = 31, 50 do
		local string = QuestWatchFrame:CreateFontString("QuestWatchLine" .. i,"BACKGROUND","QuestWatchFontTemplate")
		string:SetPoint("TOPLEFT", getglobal("QuestWatchLine" .. (i-1)), "BOTTOMLEFT")
	end
	
	self:Hook("QuestWatch_Update","Hooks_QuestWatch_Update", true)
	self:Hook("RemoveQuestWatch","Hooks_RemoveQuestWatch", true)
	self:Hook("IsQuestWatched", "Hooks_IsQuestWatched", true)
	self:Hook("GetNumQuestWatches", "Hooks_GetNumQuestWatches", true)
	self:Hook("AddQuestWatch", "Hooks_AddQuestWatch", true)
	self:Hook("GetQuestIndexForWatch", "Hooks_GetQuestIndexForWatch", true)
	self:Hook("UIParent_ManageFramePositions","Hooks_UIParent_ManageFramePositions", true)  

    self:RegisterEvent("CHAT_MSG_SYSTEM")
    self:RegisterEvent("QUEST_FINISHED")
    self:RegisterEvent("QUEST_WATCH_UPDATE")
    self:RegisterEvent("UNIT_QUEST_LOG_CHANGED")
    self:RegisterEvent("QUESTLOG_CHANGED")

	if beql.db.char.saved.qtrackercorner and beql.db.char.saved.qtrackerposx and beql.db.char.saved.qtrackerposy then
		QuestWatchFrame:ClearAllPoints()
		QuestWatchFrame:SetPoint(beql.db.char.saved.qtrackercorner,"UIParent","BOTTOMLEFT", beql.db.char.saved.qtrackerposx, beql.db.char.saved.qtrackerposy)
	else
		QuestWatchFrame:ClearAllPoints()
		QuestWatchFrame:SetPoint("TOPLEFT","UIParent","Center",0,0)
		beql:SavePositionTracker()
	end

	QuestWatchFrame:SetFrameStrata("BACKGROUND")
	QuestWatchFrame:SetMovable(true)
	QuestWatchFrame:EnableMouse(true)	
	QuestWatchFrame:SetScript("OnDragStart",function() QuestWatchFrame:StartMoving() end)
	QuestWatchFrame:SetScript("OnDragStop",function() QuestWatchFrame:StopMovingOrSizing() beql:SavePositionTracker() end)	
	
	beql.TrackerBackdrop = CreateFrame("Frame","QuestWatchFrameBackdrop",QuestWatchFrame)
	beql.TrackerBackdrop:ClearAllPoints()
	if IsAddOnLoaded("Mobmap") then
		beql.TrackerBackdrop:SetPoint("TOPLEFT", QuestWatchFrame, "TOPLEFT",-20, 0)
		beql.TrackerBackdrop:SetPoint("BOTTOMLEFT", QuestWatchFrame, "BOTTOMLEFT",-20,-40)
	else
		beql.TrackerBackdrop:SetPoint("TOPLEFT", QuestWatchFrame ,"TOPLEFT", -8, 0)
		beql.TrackerBackdrop:SetPoint("BOTTOMLEFT",QuestWatchFrame, "BOTTOMLEFT",-8,-40)
	end
	beql.TrackerBackdrop:SetPoint("TOPRIGHT", QuestWatchFrame ,"TOPRIGHT", 0, 0)	
	beql.TrackerBackdrop:SetPoint("BOTTOMRIGHT",QuestWatchFrame, "BOTTOMRIGHT", 0,-40)	
	beql.TrackerBackdrop:SetFrameStrata("BACKGROUND")
	beql.TrackerBackdrop:SetBackdrop({
		bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = true, edgeSize = 16, tileSize = 16,
		insets = { left= 5, right = 5, top = 5, bottom = 5 }
		})
	beql.TrackerBackdrop:SetFrameLevel("1")
	beql.TrackerBackdrop:Show()	
	
	beql.TrackerMinimizeButton = CreateFrame("Button","TrackerMinimizeButton", QuestWatchFrame, "UIPanelButtonTemplate")
	beql.TrackerMinimizeButton:SetNormalTexture("Interface\\AddOns\\beql\\Images\\minimize_up")
	beql.TrackerMinimizeButton:SetPushedTexture("Interface\\AddOns\\beql\\Images\\minimize_down")
	beql.TrackerMinimizeButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	beql.TrackerMinimizeButton:SetWidth(24)
	beql.TrackerMinimizeButton:SetHeight(24)
	beql.TrackerMinimizeButton:Show()
	beql.TrackerMinimizeButton:ClearAllPoints()
	beql.TrackerMinimizeButton:SetPoint("BOTTOM",QuestWatchFrame, "TOPRIGHT", -18,2)
	beql.TrackerMinimizeButton:SetScript("OnClick", function() 
		if beql.db.char.saved.minimizedtracker then 
			beql:TrackerMaximize() 
		else 
			beql:TrackerMinimize() 
		end 
	end)


	beql.TrackerTitleBar = CreateFrame("Frame","TrackerTitleBar", QuestWatchFrame)
	beql.TrackerTitleBar:ClearAllPoints()
	beql.TrackerTitleBar:SetHeight(32)
	if IsAddOnLoaded("Mobmap") then
		beql.TrackerTitleBar:SetPoint("TOPLEFT",QuestWatchFrame, "TOPLEFT",-20,25)
	else
		beql.TrackerTitleBar:SetPoint("TOPLEFT",QuestWatchFrame, "TOPLEFT",-8,25)
	end
	beql.TrackerTitleBar:SetPoint("TOPRIGHT",QuestWatchFrame, "TOPRIGHT",0,25)
	beql.TrackerTitleBar:SetBackdrop({
		bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = true, edgeSize = 16, tileSize = 16,
		insets = { left= 5, right = 5, top = 5, bottom = 5 }
		})			
	beql.TrackerTitleBar:SetFrameStrata("BACKGROUND")
	beql.TrackerTitleBar:SetFrameLevel("1")
	beql.TrackerTitleBar:SetAlpha(50)
	beql.TrackerTitleBar:SetMovable(true)
	beql.TrackerTitleBar:EnableMouse(true)
	beql.TrackerTitleBar:SetScript("OnDragStart",function() QuestWatchFrame:StartMoving() end)
	beql.TrackerTitleBar:SetScript("OnDragStop",function() QuestWatchFrame:StopMovingOrSizing() beql:SavePositionTracker() end)		
	beql.TrackerTitleBar:Show()
	
	beql.TrackerTitleBarText = beql.TrackerTitleBar:CreateFontString("TrackerTitleBarText","BACKGROUND","QuestWatchFontTemplate")
	beql.TrackerTitleBarText:SetPoint("LEFT",10,1)
	
	QuestWatch_Update()
end


function beql:SavePositionTracker()
	beql:TrackerLockCornerForGrowth()
end

function beql:TrackerMinimize()
	beql.db.char.saved.minimizedtracker = true
	beql.TrackerMinimizeButton:SetNormalTexture("Interface\\AddOns\\beql\\Images\\restore_up")
	beql.TrackerMinimizeButton:SetPushedTexture("Interface\\AddOns\\beql\\Images\\restore_down")	
	for i = 1, 50 do
		local CurFrame = getglobal("QuestWatchLine"..(i))
		CurFrame:Hide()
	end	
	local minWidth = beql.TrackerTitleBarText:GetWidth()+32
	QuestWatchFrame:SetWidth(minWidth)
	beql.TrackerBackdrop:Hide()
end

function beql:TrackerMaximize()
	beql.db.char.saved.minimizedtracker = false
	beql.TrackerMinimizeButton:SetNormalTexture("Interface\\AddOns\\beql\\Images\\minimize_up")
	beql.TrackerMinimizeButton:SetPushedTexture("Interface\\AddOns\\beql\\Images\\minimize_down")	
	for i = 1, 50 do
		local CurFrame = getglobal("QuestWatchLine"..(i))
		CurFrame:Show()
	end	
	beql.TrackerBackdrop:Show()
	QuestWatch_Update()
end

--
-- Events
--

function beql:CHAT_MSG_SYSTEM(arg1)
	if beql.db.profile.AddNew then
		if string.find(arg1, BEQL_QUEST_ACCEPTED.." .+") then
			local temp = string.gsub(arg1, BEQL_QUEST_ACCEPTED.." ", "")
			beql.TrackerQueue = temp
			beql:ManageQuests()
		end
	end
end

function beql:QUEST_FINISHED()
	beql:ManageQuests()
	QuestWatch_Update()
end

function beql:QUEST_WATCH_UPDATE()
	beql:ManageQuests()
	QuestWatch_Update()
end

function beql:UNIT_QUEST_LOG_CHANGED()
	beql:ManageQuests()
	QuestWatch_Update()
end

function beql:QUESTLOG_CHANGED()
	beql:ManageQuests()
	QuestWatch_Update()
end

--
-- Hooks
--

function beql:Hooks_AddQuestWatch(questIndex)
	local questName, level = GetQuestLogTitle(questIndex)
	local questLogHeader, tempId

	if IsQuestWatched(questIndex) then
		return
	else
		isHeader = false
		tempId = questIndex
		while (not isHeader) do
			questLogHeader, _, _, _, isHeader = GetQuestLogTitle(tempId)
			tempId = tempId-1
		end

		table.insert(beql.db.char.QuestWatches, questLogHeader..","..level..","..questName)
	end
	beql:SortWatchedQuests()
end

function beql:Hooks_RemoveQuestWatch(questIndex)
	local questName, level = GetQuestLogTitle(questIndex)

	local questLogHeader, isHeader, tempId

	isHeader = false
	tempId = questIndex
	while (not isHeader) do
		questLogHeader, _, _, _, isHeader = GetQuestLogTitle(tempId)
		tempId = tempId-1
	end

	local temp = questLogHeader..","..level..","..questName

	if table.getn(beql.db.char.QuestWatches) > 0 then
		for i=1, table.getn(beql.db.char.QuestWatches) do
			if beql.db.char.QuestWatches[i] == temp then
				table.remove(beql.db.char.QuestWatches , i)
				break
			end
		end
	end
	beql:SortWatchedQuests()
end

function beql:Hooks_IsQuestWatched(questIndex)
	local questName, level = GetQuestLogTitle(questIndex)
	local questLogHeader, isHeader, tempId

	isHeader = false
	tempId = questIndex
	while (not isHeader and tempId > 0) do
		questLogHeader, _, _, _, isHeader = GetQuestLogTitle(tempId)
		tempId = tempId-1
	end

	local temp = questLogHeader..","..level..","..questName
	if table.getn(beql.db.char.QuestWatches) > 0 then
		for i=1, table.getn(beql.db.char.QuestWatches) do
			if beql.db.char.QuestWatches[i] == temp then
				return true
			end
		end
	end
	return false
end


-- Returns the number of watched quests
function beql:Hooks_GetNumQuestWatches()
	return table.getn(beql.db.char.QuestWatches)
end

function beql:Hooks_GetQuestIndexForWatch(id)
	local numEntries = GetNumQuestLogEntries()
	local questLogTitleText, level
	local questLogHeader, isHeader, tempId
	local questFound = false
	local temp, currentHeader=nil

	for i=1, numEntries, 1 do
		questLogTitleText, level, _, _, isHeader, _ = GetQuestLogTitle(i)
		if isHeader then
			currentHeader = questLogTitleText
		else
			temp = currentHeader..","..level..","..questLogTitleText
			if  temp == beql.db.char.QuestWatches[id]  then
				return i
			end
		end
	end
	return 0
end

function beql:Hooks_UIParent_ManageFramePositions()
	self.hooks.UIParent_ManageFramePositions()
	if beql.db.char.saved.qtrackercorner and beql.db.char.saved.qtrackerposx and beql.db.char.saved.qtrackerposy then
		QuestWatchFrame:ClearAllPoints()
		QuestWatchFrame:SetPoint(beql.db.char.saved.qtrackercorner,"UIParent","BOTTOMLEFT", beql.db.char.saved.qtrackerposx, beql.db.char.saved.qtrackerposy)
	end	
end

function beql:Hooks_QuestWatch_Update()

	if beql.db.profile.ShowTrackerHeader then
		t1, _, t2 = QuestWatchLine1:GetFont()
		beql.TrackerTitleBarText:SetFont(t1, beql.db.profile.TrackerFontHeight+2, t2)
		beql.TrackerTitleBarText:SetHeight(beql.db.profile.TrackerFontHeight+2)	
		beql.TrackerTitleBarText:Show()
		local questnums = GetNumQuestWatches()
		local completedquests = beql:GetCompletedQuestNum()
		beql.TrackerTitleBarText:SetText(L["Quest Tracker"].. " ("..questnums - completedquests ..")")		
		beql.TrackerMinimizeButton:Show()
		beql.TrackerTitleBar:Show()
	else
		beql.TrackerTitleBar:Hide()
		beql.TrackerMinimizeButton:Hide()
		if beql.db.char.saved.minimizedtracker then
			beql:TrackerMaximize()
		end		
	end
	
	if beql.db.profile.CustomBackgroundColor then
		beql.TrackerBackdrop:SetBackdropColor(beql.db.profile.Color.BackgroundColor.r,beql.db.profile.Color.BackgroundColor.g,beql.db.profile.Color.BackgroundColor.b)
		beql.TrackerBackdrop:SetBackdropBorderColor(beql.db.profile.Color.BackgroundCornerColor.r,beql.db.profile.Color.BackgroundCornerColor.g,beql.db.profile.Color.BackgroundCornerColor.b)
		beql.TrackerTitleBar:SetBackdropColor(beql.db.profile.Color.BackgroundColor.r,beql.db.profile.Color.BackgroundColor.g,beql.db.profile.Color.BackgroundColor.b)
		beql.TrackerTitleBar:SetBackdropBorderColor(beql.db.profile.Color.BackgroundCornerColor.r,beql.db.profile.Color.BackgroundCornerColor.g,beql.db.profile.Color.BackgroundCornerColor.b)		
		if beql.db.profile.BackgroundAlpha then
			beql.TrackerBackdrop:SetAlpha(50)
		else
			beql.TrackerBackdrop:SetAlpha(0)
			beql.TrackerTitleBar:SetBackdropColor(0,0,0,0)
			beql.TrackerTitleBar:SetBackdropBorderColor(0,0,0,0)				
		end
	else
		beql.TrackerBackdrop:SetBackdropBorderColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
		beql.TrackerBackdrop:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
		beql.TrackerTitleBar:SetBackdropBorderColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
		beql.TrackerTitleBar:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)					
		if beql.db.profile.BackgroundAlpha then
			beql.TrackerBackdrop:SetAlpha(50)
		else
			beql.TrackerBackdrop:SetAlpha(0)								
			beql.TrackerTitleBar:SetBackdropColor(0,0,0,0)	
			beql.TrackerTitleBar:SetBackdropBorderColor(0,0,0,0)		
		end
	end	
	
	--------------------------------------------
	-- Original QuestWatch_Update code (altered)
	--------------------------------------------
	
	local numObjectives
	local questWatchMaxWidth = 0
	local tempWidth
	local watchText
	local text, type, finished
	local questTitle
	local watchTextIndex = 1
	local questIndex
	local objectivesCompleted
	local qwHeight = 12
	local questLogTitleText, isComplete, isCompleted, isRemoved, level
	local currentHeader = nil
	local temp, isCollapsed, isOk	
	local tempColor, tempColor2, tempColor3
	local tempArray = {}, tempI
	local tempObj, tempDone, tempLevel, tempTag	

	for i=1, GetNumQuestWatches() do
		isRemoved = false
		questIndex = GetQuestIndexForWatch(i)
		if  questIndex  then
			_, level, _, _, _, _, isCompleted = GetQuestLogTitle(questIndex)
			numObjectives = GetNumQuestLeaderBoards(questIndex)
		
			-- Show Zone names
			temp, isCollapsed = beql:GetQuestHeaderForWatch(i)
			if currentHeader == nil or currentHeader ~= temp then
				currentHeader = temp
				if beql.db.profile.ShowZonesInTracker then
					watchText = getglobal("QuestWatchLine"..watchTextIndex)
					if watchText ~= nil then
						t1, _, t2 = watchText:GetFont()
						watchText:SetText(currentHeader)
						watchText:SetFont(t1, beql.db.profile.TrackerFontHeight, t2)
						watchText:SetHeight(beql.db.profile.TrackerFontHeight+1)						
						tempWidth = watchText:GetWidth()
						-- Set the anchor of the title line a little lower
						if ( watchTextIndex > 1 ) then
							watchText:SetPoint("TOPLEFT", "QuestWatchLine"..(watchTextIndex - 1), "BOTTOMLEFT", 0, -4)
							qwHeight = qwHeight+4
						end
						if beql.db.profile.CustomZoneColor then
							watchText:SetTextColor(beql.db.profile.Color.Zone.r, beql.db.profile.Color.Zone.g, beql.db.profile.Color.Zone.b)
						else
							watchText:SetTextColor(1, 1, 1)
						end
						watchText:Show()
						if  tempWidth > questWatchMaxWidth  then
							questWatchMaxWidth = tempWidth
						end
					end
					watchTextIndex = watchTextIndex + 1
				end
			end		

			if beql.db.profile.RemoveFinished and isCompleted then
				isRemoved = true
			end		
		
		
			if isCollapsed == nil and questIndex > 0 and not isRemoved then
				questLogTitleText, tempLevel, tempTag, _, _, _, isComplete = GetQuestLogTitle(questIndex)
				watchText = getglobal("QuestWatchLine"..watchTextIndex)
				if watchText ~= nil and questLogTitleText ~= nil then
					watchText:SetText("  "..questLogTitleText)
					t1, _, t2 = watchText:GetFont()
					watchText:SetFont(t1, beql.db.profile.TrackerFontHeight, t2)
					watchText:SetHeight(beql.db.profile.TrackerFontHeight+1)						
					tempWidth = watchText:GetWidth();
					-- Set the anchor of the title line a little lower
					if  watchTextIndex > 1  then
					 	watchText:SetPoint("TOPLEFT", "QuestWatchLine"..(watchTextIndex - 1), "BOTTOMLEFT", 0, -4)
						qwHeight = qwHeight+4
					end
					watchText:Show()
					if  tempWidth > questWatchMaxWidth then
						questWatchMaxWidth = tempWidth
					end
					
				end				
				watchTextIndex = watchTextIndex + 1

				if beql.db.profile.MinimizeFinished and isCompleted then
					numObjectives = 0
				end				

				local markerID = 0
				--If there are objectives set the title
				if  numObjectives > 0 then
					objectivesCompleted = 0
					tempObj = 0
					tempDone = 0
					for j=1, numObjectives do
						text, type, finished = GetQuestLogLeaderBoard(j, questIndex)
						if finished and beql.db.profile.MinimizeFinished then
							objectivesCompleted = objectivesCompleted + 1
						else
							watchText = getglobal("QuestWatchLine"..watchTextIndex)
							t1, _, t2 = watchText:GetFont()
							watchText:SetFont(t1, beql.db.profile.TrackerFontHeight, t2)
							watchText:SetHeight(beql.db.profile.TrackerFontHeight+1)						
							-- Set Objective text
							if  beql.db.profile.ShowObjectiveMarkers  then
								if beql.db.profile.UseTrackerListing then 
									watchText:SetText("   "..BEQL_TrackerLists[beql.db.profile.TrackerList][markerID]..") "..text)
								else
									watchText:SetText("   "..BEQL_TrackerSymbols[beql.db.profile.TrackerSymbol].." "..text)
								end
							else
								watchText:SetText("   "..text)
							end							
							-- Color the objectives
							if beql.db.profile.CustomObjetiveColor then
									tempColor = { r=beql.db.profile.Color.ObjectiveEmpty.r, g=beql.db.profile.Color.ObjectiveEmpty.g, b=beql.db.profile.Color.ObjectiveEmpty.b }
									tempColor2 = { r=beql.db.profile.Color.ObjectiveComplete.r, g=beql.db.profile.Color.ObjectiveComplete.g, b=beql.db.profile.Color.ObjectiveComplete.b }
							else
									tempColor = {r=0.8, g=0.8, b=0.8}
									tempColor2 = {r=HIGHLIGHT_FONT_COLOR.r, g=HIGHLIGHT_FONT_COLOR.g, b=HIGHLIGHT_FONT_COLOR.b}
							end
							
							
							
							if ( finished ) then
								watchText:SetTextColor(tempColor2.r, tempColor2.g, tempColor2.b)
								objectivesCompleted = objectivesCompleted + 1
							else
								tempI = 0
								tempArray = {}
								for v in string.gmatch(text, "%d+") do
									tempI = tempI+1
									table.insert (tempArray, v)
								end
								if tempI == 0 then
									tempI = 2
									tempArray[1] = "0"
									tempArray[2] = "1"
								 elseif tempI == 1 then
									tempI = 2
									tempArray[2] = 99999
								end
								tempObj  = tempObj + tempArray[tempI]
								tempDone = tempDone + tempArray[tempI-1]							
								if beql.db.profile.FadeColor then
									tempColor3 = beql:FadeColors(tempColor, tempColor2, tempArray[tempI-1], tempArray[tempI])
								else
									tempColor3 = tempColor
								end
								watchText:SetTextColor(tempColor3.r, tempColor3.g, tempColor3.b)
							end

							
							
							tempWidth = watchText:GetWidth()
							if  tempWidth > questWatchMaxWidth  then
								questWatchMaxWidth = tempWidth
							end
							watchText:SetPoint("TOPLEFT", "QuestWatchLine"..(watchTextIndex - 1), "BOTTOMLEFT", 0, 0)
							watchText:Show()							
							watchTextIndex = watchTextIndex + 1				
							markerID = markerID+1	
						end
					end
				else
					tempObj = 1
					tempDone = 0
					numObjectives = 0
				end
				-- Brighten the quest title if all the quest objectives were met
				watchText = getglobal("QuestWatchLine"..watchTextIndex-markerID-1)
				if  (objectivesCompleted == numObjectives) then
					if beql.db.profile.CustomHeaderColor then
						watchText:SetTextColor(beql.db.profile.Color.HeaderComplete.r, beql.db.profile.Color.HeaderComplete.g, beql.db.profile.Color.HeaderComplete.b)
					else
						watchText:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
					end
				else
					if beql.db.profile.CustomHeaderColor then
						if beql.db.profile.HeaderQuestLevelColor then
							watchText:SetTextColor(GetDifficultyColor(tempLevel).r,GetDifficultyColor(tempLevel).g,GetDifficultyColor(tempLevel).b)
						else
							watchText:SetTextColor(beql.db.profile.Color.HeaderEmpty.r, beql.db.profile.Color.HeaderEmpty.g, beql.db.profile.Color.HeaderEmpty.b)
						end
					else
						watchText:SetTextColor(0.75, 0.61, 0)
					end
				end
				
			end
		end
	end

	-- Set tracking indicator
	if ( GetNumQuestWatches() > 0 ) then
		QuestLogTrackTracking:SetVertexColor(0, 1.0, 0)
	else
		QuestLogTrackTracking:SetVertexColor(1.0, 0, 0)
	end
	
	-- If no watch lines used then hide the frame and return
	if ( watchTextIndex == 1 ) then
		QuestWatchFrame:Hide()
		return
	else
		QuestWatchFrame:Show()
		QuestWatchFrame:SetHeight(watchTextIndex * beql.db.profile.TrackerFontHeight+1 + qwHeight)
		QuestWatchFrame:SetWidth(questWatchMaxWidth + 12)
		local minWidth = beql.TrackerTitleBarText:GetWidth()+32
		local curWidth = QuestWatchFrame:GetWidth()
		if curWidth < minWidth then
			QuestWatchFrame:SetWidth(minWidth)
		end		
	end

	-- Hide unused watch lines
	for i=watchTextIndex, MAX_QUESTWATCH_LINES do
		getglobal("QuestWatchLine"..i):Hide();
	end
	UIParent_ManageFramePositions()
	--QuestWatchFrameBackdrop:SetPoint("TOPLEFT", "QuestWatchFrame", "TOPLEFT", -8, -4)
	
	if beql.db.char.saved.minimizedtracker then
		beql:TrackerMinimize()
	end
	
	beql:TrackerLockCornerForGrowth()
end

function beql:TrackerLockCornerForGrowth()
	local Left = QuestWatchFrame:GetLeft()
	local Right = QuestWatchFrame:GetRight()
	local Top = QuestWatchFrame:GetTop()
	local Bottom = QuestWatchFrame:GetBottom()
	local lock
	local pointone
	local pointtwo
	local TOPBOTTOM_MEDIAN = 384
	local LEFTRIGHT_MEDIAN = 512
	if Left and Right and Top and Bottom then
		if Bottom < TOPBOTTOM_MEDIAN and Top > TOPBOTTOM_MEDIAN then
			local topcross = Top - TOPBOTTOM_MEDIAN
			local bottomcross = TOPBOTTOM_MEDIAN - Bottom
			if bottomcross > topcross then
				lock = "BOTTOM"
				pointtwo = Bottom
			else
				lock = "TOP"
				pointtwo = Top
			end
		elseif Top > TOPBOTTOM_MEDIAN then
			lock = "TOP"
			pointtwo = Top
		elseif Bottom < TOPBOTTOM_MEDIAN then
			lock = "BOTTOM"
			pointtwo = Bottom
		end
		if Left < LEFTRIGHT_MEDIAN and Right > LEFTRIGHT_MEDIAN then
			local leftcross = LEFTRIGHT_MEDIAN - Left
			local rightcross = Right - LEFTRIGHT_MEDIAN
			if rightcross > leftcross then
				lock = lock.."RIGHT"
				pointone = Right
			else
				lock = lock.."LEFT"
				pointone = Left
			end
		elseif Left < LEFTRIGHT_MEDIAN then
			lock = lock.."LEFT"
			pointone = Left
		elseif Right > LEFTRIGHT_MEDIAN then
			lock = lock.."RIGHT"
			pointone = Right
		end
		if lock and lock ~= "" and pointone and pointtwo then
			QuestWatchFrame:ClearAllPoints()
			QuestWatchFrame:SetPoint(lock,"UIParent","BOTTOMLEFT",pointone,pointtwo)
			beql.db.char.saved.qtrackercorner = lock
			beql.db.char.saved.qtrackerposx = pointone
			beql.db.char.saved.qtrackerposy = pointtwo
		elseif beql.db.char.saved.qtrackercorner and beql.db.char.saved.qtrackerposx and beql.db.char.saved.qtrackerposy then
			QuestWatchFrame:ClearAllPoints()
			QuestWatchFrame:SetPoint(beql.db.char.saved.qtrackercorner,"UIParent","BOTTOMLEFT",beql.db.char.saved.qtrackerposx,beql.db.char.saved.qtrackerposy)
		end
	end
end

function beql:GetCompletedQuestNum()
	local level,isCompleted,questIndex
	local num
	num = 0
	for i=1, GetNumQuestWatches() do
				questIndex = GetQuestIndexForWatch(i)
				_, level, _, _, _, _, isCompleted = GetQuestLogTitle(questIndex)
				if isCompleted and beql.db.profile.RemoveFinished then
					num = num + 1
				end
	end
	return num
end